/*******************************************************************************
 * Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2010, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.common.web.ui.util"); //$NON-NLS-1$

/**
 * Common util functions that don't have a better home.
 */
(function(){

function returnSameObject(object) {
	return object;
}

var dummyNode = dojo.create('div'); //$NON-NLS-1$

dojo.mixin(com.ibm.relm.common.web.ui.util, {

	/**
	 * Centers a DOM node within a scrollable parent.
	 * 
	 * @param  {object} args args.scrollView the parent DOM node to scroll.
	 *                       args.node the node to center.
	 *                       args.horizontally true to center horizontally. Otherwise, don't change horizontal scroll.
	 *                       args.vertically true center vertically. Otherwise, don't change vertical scroll.
	 *                       args.force will always center, even if already visible.
	 */
	centerInView: function(args) {

		function valueInRange(value, rangeStart, rangeEnd) {
			return value >= rangeStart && value <= rangeEnd;
		}

		var scrollView = args.scrollView;
		var node = args.node;

		var scrollViewPos = dojo.position(scrollView, true);
		var nodePos = dojo.position(node, true);

		var nodeTopOffset = nodePos.y - scrollViewPos.y + scrollView.scrollTop;
		var nodeLeftOffset = nodePos.x - scrollViewPos.x + scrollView.scrollLeft;
	
		// XXX: this isn't working right with product tree... doesn't still can be scrolled off, but seems perfect with history view.
		// Deferring looking into this into after more product tree changes.
		if (args.vertically) {
			if (args.force || !valueInRange(nodeTopOffset, scrollView.scrollTop, scrollView.scrollTop + scrollViewPos.h)) {
				scrollView.scrollTop = nodeTopOffset - scrollViewPos.h / 2 + nodePos.h / 2;
			}
		}

		if (args.horizontally) {
			scrollView.scrollLeft = nodeLeftOffset - scrollViewPos.w / 2 + nodePos.w / 2;			
		}
	},

	/**
	 * Fades background color of a node from orange to white. Used to highlight
	 * new nodes.
	 * @param node domNode
	 * @param args.endColor color to fade to. Defaults to white. Empty background color is restored when done.
	 * @param args.duration cutom duration. Default is one second
	 */
	flashNode: function(node, args) {
		args = args || {};
		var endColor = args.endColor || 'white'; //$NON-NLS-1$
		var anim = dojo.animateProperty({
			node: node,
			properties: {
				'backgroundColor': {start: '#FECC7E', end: endColor} //$NON-NLS-1$ //$NON-NLS-2$
			},
			duration: args.duration || 1000,
			onEnd: function() { 
				dojo.style(node, 'backgroundColor', ''); //$NON-NLS-1$ //$NON-NLS-2$
			}
		});
		anim.play();
		return anim;
	},

	setText: function(node, text) {
		dojo.empty(node);
		node.appendChild(
			node.ownerDocument.createTextNode(text));
	},

	/**
	 * Checks if a mouse event is within the bounds of a position region.
	 * @param  {mouseevent}  e   [description]
	 * @param  {object}  pos must have properties for x, y, w, and h.
	 * @param {number} vOffset a vertical offset to shift the checked position region by.
	 * Used then the region is in a scroll container. This would be the number of pixels
	 * the contatiner scrolled since the original position was cached. This is faster
	 * that recalculating  the position all the time. Can be undefined.
	 * @return {Boolean}     true if over the region.
	 */
	isEventWithin: function(e, pos, vOffset) {
		vOffset = vOffset || 0;
		if (e.clientX >= pos.x && e.clientY >= pos.y + vOffset) {
			var right = pos.x + pos.w;
			var bottom = pos.y + pos.h + vOffset;
			if (e.clientX <= right && e.clientY <= bottom) {
				return true;
			}
		}

		return false;
	},

	/**
	 * Utility method to download a file using the standard method used throughout RELM. A hidden form is cretaed
	 * with a URL to a RELM service which returns a file to download. 
	 * 
	 * @param  {String} url       	The URL to the RELM service.
	 * @param  {String} fieldName 	The name of the hidden form field the server expects
	 * @param  {Array} values     	Must be an array of values. The array of values are all added
	 *                             		as a hidden form field with the name provided in fieldName
	 * @param  {boolean} newWindow 	If true, then the form's "target" is set "_blank"
	 * @param  {boolean} useGet		If true, then the form uses a get or else a post is used. 
	 */
	downloadFile: function(url, fieldName, values, newWindow, useGet) {
		var formArgs = {};
		formArgs.action = (url.indexOf(net.jazz.ajax._contextRoot) === 0) ?
			url : net.jazz.ajax._contextRoot + url;
		formArgs.method = (useGet) ? 'get' : 'post'; //$NON-NLS-1$ //$NON-NLS-2$

		// Targeting _blank shows an ugly popup, but without it, the
		// user may get a warning that the Query isn't saved since the
		// browser thinks the POST to the current page could navigate
		// away. In practice, it's always a file download, so it
		// won't, but there's no way to indicate that until after the
		// download starts.
		if (newWindow) formArgs.target = "_blank"; //$NON-NLS-1$

		var downloadForm = dojo.create('form', formArgs, document.body); //$NON-NLS-1$

		for (var i = 0; i < values.length; i++) {
			var value = values[i];
			var input = dojo.create('input', { //$NON-NLS-1$
				name: fieldName,
				type: 'hidden', //$NON-NLS-1$
				value: value
			}, downloadForm);
		}

		downloadForm.submit();
		downloadForm.parentNode.removeChild(downloadForm);
	},
	
	stripHtml: function(text) {
		dummyNode.innerHTML = text;
		return dummyNode.textContent;
	},

	/**
	 * Escape HTML for RELM config.
	 * Text in RELM config is escaped in ViewConfigJSONUtil.java in 6.0.1, 
	 * but old config may contain "<" and ">" characters which should be escaped. 
	 */
	escapeHtml: function(text){
		if(!dojo.isString(text)) return text;
		return text.replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/\"/g,"&quot;"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},

	unescapeHtml: function(text){
		if(!dojo.isString(text)) return text;
		return text.replace(/&lt;/g,"<").replace(/&gt;/g,">").replace(/&quot;/g,"\""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	},

	/**
	 * Calculates the "best" x, y coordinate to place a menu at when showing on
	 * screen. This will be relative to either a DOM node or click event.
	 * Normally the position will below that "around" thing, but if the menu
	 * would overflow the viewport it will be shifted up. Attempts are made not
	 * to position the menu underneath the cursor that no menu items are pre-
	 * highlighted when the menu opens (though this is unavoidable in very
	 * small viewports.)
	 *
	 * This serves as a replacement for the `around` parameter to
	 * dijit.popup.open. That will often position a menu so some of its items
	 * are truncated (overflow the viewport), especially with large menus like
	 * in the product tree. This will works with that function, but the results
	 * of this function should be set as the explicit .x and .y parameters to
	 * the popup-args instead of trying to use .around.
	 * 
	 * @param  {Widget} menu              normally a menu widget, but anything
	 *     with a .domNode property will do. The node *must* be already added
	 *     to the DOM or this will fail. This is required for accurate sizing
	 *     of the menu. In most cases you can add to document.body, then have
	 *     dijit.popup.open move the menu into the final position.
	 * @param  {DOM node or mouse event} aroundNodeOrEvent the thing to
	 *     position the menu around. This can either be a DOM node, like that
	 *     for a button; or a mouseevent with clickX and clickY properties.
	 * @return {object}                   object with .x and .y properties as
	 *     integers in pixels. Like {x: 100, y: 200}. Can be mixin-ed to popup
	 *     args.
	 */
	calculateMenuPosition: function(menu, aroundNodeOrEvent) {
		// Must be in the the DOM for accurate size calculations. While this
		// could add and remove, it's probably faster to have the caller be
		// required to just put it in for us to avoid thrashing.
		var menuNode = menu.domNode;
		if (!menuNode.parentNode) {
			throw new Error('A must be added to the DOM before it can be positioned.'); //$NON-NLS-1$
		}
		var menuPos = dojo.position(menuNode);

		// Work "around" a node or a click event. Duck-type to check for click
		// event with a clientX. In that case, the around position is
		// manufactured around the click with a fake height and width of 1px
		// where the user clicked. Otherwise, assume this is a dom node and get
		// it real position.
		var aroundPos;
		if (!aroundNodeOrEvent.clientX) {
			aroundPos = dojo.position(aroundNodeOrEvent);
		}
		else {
			aroundPos = {
				x: aroundNodeOrEvent.clientX,
				y: aroundNodeOrEvent.clientY,
				h: 0,
				w: 0
			};
		}

		// Assume menu is left aligned with around/button
		// and positioned underneath it.
		var menuX = aroundPos.x;
		var menuY = aroundPos.y + aroundPos.h;

		var viewport = dijit.getViewport();
		var menuBottom = menuY + menuPos.h;
		var viewportOverflowY = menuBottom - viewport.h;

		var maxAllowedOverflow = 10;

		if (viewportOverflowY > maxAllowedOverflow) {
			// Pull menu up, so it doesn't flow off the page.
			menuY -= viewportOverflowY;

			// Shift to the right of the 'around' button so it doesn't open under
			// the cursor now the we know it's not "below" but around-button anymore.
			menuX += aroundPos.w + 1;
		}

		var menuRight = menuX + menuPos.w;
		var viewportOverflowX = menuRight - viewport.w;
		if (viewportOverflowX > maxAllowedOverflow) {
			// Now check that we don't overflow out of the right of the page.
			// Position to the left of around node in that case.
			menuX = aroundPos.x - menuPos.w;
		}

		return {
			x: menuX,
			y: menuY
		};
	}

});
})();