/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.Pinning'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui', 'PinningMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization('com.ibm.relm.ple.web.rtc', 'StreamManagementMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.Promises'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

(function() {
	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui', 'PinningMessages'); //$NON-NLS-1$ //$NON-NLS-2$

	var productapi = com.ibm.relm.ple.web.ui.productapi;
	var StreamManagement = com.ibm.relm.ple.web.rtc.StreamManagement;
	var TreeSelection = com.ibm.relm.ple.web.ui.TreeSelection;
	var Promises = com.ibm.relm.common.web.Promises;

	/**
	 * Calls the server to actually set a new pinned mode then refreshes tree.
	 * 	
	 * @param  {site} site       the site to pin. Assumed to be bound item
	 *     version. Will use the version of branch URI of this depednding on
	 *     the pinned mode.
	 * @param  {string} pinnedMode pinned mode to set. Either 'versionPinned',
	 *     'branchPinned', or null to unpin. "Branch pinned while mutable" is
	 *     only set implicitly by API operations.
	 * @return {Deferred} fires once operation is done with empty callback or
	 *     error object on errback.
	 */
	function doSetPinnedMode(site, pinnedMode) {
		// We set the item version, branch, or nothing depending on if
		// we're pinning one of those or unpinning.
		var itemVersionUri;
		var branchUri;

		var message;

		if (pinnedMode === 'versionPinned') { //$NON-NLS-1$
			message = messages.pinning;
			itemVersionUri = site.itemVersion.uri;
		}
		else if (pinnedMode === 'branchPinned') { //$NON-NLS-1$
			message = messages.pinning;
			branchUri = site.itemVersion.branchUri;
		}
		else {
			message = messages.unpinning;
			// Unpin. Set nothing.
			itemVersionUri = null;
			branchUri = null;
		}

		var deferred = new dojo.Deferred();
		var statusMessage = com.ibm.relm.common.web.StatusMessage.show(message);

		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/ple/pin', //$NON-NLS-1$
			content: {
				contextConfigVersionUri: site.contextConfigVersionUri,
				contextPartUri: site.contextPartUri,
				itemVersionUri: itemVersionUri,
				branchUri: branchUri
			},

			load: function () {
				com.ibm.relm.ple.web.ui.tree.TreeRefresh.refresh({
					selectPath: site.path
				}).then(function() {
					deferred.callback();
				});
			},

			error: function(error, ioArgs) {
				var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
				deferred.errback(errorObject);
			},

			handle: function() {
				statusMessage.hide();
			}
		});

		return deferred;
	}

	/**
	 * Prompts the user to unpin the item version in the given site.
	 * Unpins if they OK it, then refreshes the tree.
	 * @param  {site} site site to unpin.
	 */
	function doConfirmAndUnpin(site) {
		var title = com.ibm.relm.ple.web.ui.formatting.formatProductTitleForItem(site.itemVersion);

		var d = new com.ibm.relm.ple.web.ui.pages.ConfirmDialog({
			dialogTitle: messages.unpinTitle,
			prompt: dojo.replace(messages.unpinPrompt, [com.ibm.relm.common.web.ui.util.escapeHtml(title)]),
			details:  messages.unpinDetailMessage,
			iconClass: 'ple-icon-unpinned' //$NON-NLS-1$
		});

		d.addButton(messages.unpinAction, function() {
			return doSetPinnedMode(site, null);
		}, true);

		d.startup();
	}

	/**
	 * Confirms the operation if needed, then sets the pinned mode if OK.
	 * Show either confimration or simple progress dialog while operation and refresh are occurring.
	 * 
	 * @param {site} site          the site to pin or unpin.
	 * @param  {string} pinnedMode pinned mode to set. Either 'versionPinned',
	 *     'branchPinned', or null to unpin. "Branch pinned while mutable" is
	 *     only set implicitly by API operations.
	 */
	function setPinnedMode(site, newPinnedMode) {
		if (!newPinnedMode) {
			doConfirmAndUnpin(site);
			return;
		}

		var iconClass = com.ibm.relm.ple.web.ui.tree.PinnedModeButton.getIconClassForPinnedMode(newPinnedMode);

		var progressDialog = new com.ibm.relm.ple.web.ui.ProgressDialog({
			primaryTitle: messages.pinDialogTitle, 
			explanation: '', //$NON-NLS-1$
			canCancel: false,
			mirrorStatusMessage: true,
			iconClass: iconClass
		});
		progressDialog.startup();

		Promises.later(doSetPinnedMode(site, newPinnedMode), {
			callback: function() {
				progressDialog.close();		
			},
			errback: function(errorObject) {
				progressDialog.close();
				dojo.publish('ple.product_tree_error', [errorObject]); //$NON-NLS-1$
			}
		});
	}

 /**
  * Utility to contain all pinning related logic. Performs the standard pinning operation for products,
  * but is also aware of special actions that need to happen at pinning, such as RTC component updates.
  */
dojo.mixin(dojo.getObject('com.ibm.relm.ple.web.ui.Pinning'), { //$NON-NLS-1$
	setPinnedMode: setPinnedMode
});

})();
